@@ -9,6 +9,7 @@ from typing import TYPE_CHECKING
 
 import xcffib
 import xcffib.xproto
+from xcffib.wrappers import GContextID, PixmapID
 from xcffib.xproto import EventMask, SetMode, StackMode
 
 from libqtile import hook, utils
@@ -88,6 +89,7 @@ def _geometry_getter(attr):
             self.y = g.y
             self.width = g.width
             self.height = g.height
+            self.depth = g.depth
         return getattr(self, "_" + attr)
     return get_attr
 
@@ -285,7 +287,7 @@ class XWindow:
 
     def configure(self, **kwargs):
         """
-        Arguments can be: x, y, width, height, border, sibling, stackmode
+        Arguments can be: x, y, width, height, borderwidth, sibling, stackmode
         """
         mask, values = xcbq.ConfigureMasks(**kwargs)
         # older versions of xcb pack everything into unsigned ints "=I"
@@ -311,9 +313,9 @@ class XWindow:
         """
         Parameters
         ==========
-        name : String Atom name
-        type : String Atom name
-        format : 8, 16, 32
+        name: String Atom name
+        type: String Atom name
+        format: 8, 16, 32
         """
         if name in xcbq.PropertyMap:
             if type or format:
@@ -426,9 +428,60 @@ class XWindow:
             parent = XWindow(self.conn, q.parent)
         return root, parent, [XWindow(self.conn, i) for i in q.children]
 
-    def paint_borders(self, color):
-        if color:
-            self.set_attribute(borderpixel=self.conn.color_pixel(color))
+    def paint_borders(self, depth, colors, borderwidth, width, height):
+        """
+        This method is used only by the managing Window class.
+        """
+        if not colors or not borderwidth:
+            return
+
+        if isinstance(colors, str):
+            self.set_attribute(borderpixel=self.conn.color_pixel(colors))
+            return
+
+        if len(colors) > borderwidth:
+            colors = colors[:borderwidth]
+        core = self.conn.conn.core
+        outer_w = width + borderwidth * 2
+        outer_h = height + borderwidth * 2
+
+        with PixmapID(self.conn.conn) as pixmap:
+            with GContextID(self.conn.conn) as gc:
+                core.CreatePixmap(
+                    depth, pixmap, self.wid, outer_w, outer_h
+                )
+                core.CreateGC(gc, pixmap, 0, None)
+                borders = len(colors)
+                borderwidths = [borderwidth // borders] * borders
+                for i in range(borderwidth % borders):
+                    borderwidths[i] += 1
+                coord = 0
+                for i in range(borders):
+                    core.ChangeGC(
+                        gc, xcffib.xproto.GC.Foreground, [self.conn.color_pixel(colors[i])]
+                    )
+                    rect = xcffib.xproto.RECTANGLE.synthetic(
+                        coord, coord, outer_w - coord * 2, outer_h - coord * 2
+                    )
+                    core.PolyFillRectangle(pixmap, gc, 1, [rect])
+                    coord += borderwidths[i]
+                self._set_borderpixmap(depth, pixmap, gc, borderwidth, width, height)
+
+    def _set_borderpixmap(self, depth, pixmap, gc, borderwidth, width, height):
+        core = self.conn.conn.core
+        outer_w = width + borderwidth * 2
+        outer_h = height + borderwidth * 2
+        with PixmapID(self.conn.conn) as border:
+            core.CreatePixmap(
+                depth, border, self.wid, outer_w, outer_h
+            )
+            most_w = outer_w - borderwidth
+            most_h = outer_h - borderwidth
+            core.CopyArea(pixmap, border, gc, borderwidth, borderwidth, 0, 0, most_w, most_h)
+            core.CopyArea(pixmap, border, gc, 0, 0, most_w, most_h, borderwidth, borderwidth)
+            core.CopyArea(pixmap, border, gc, borderwidth, 0, 0, most_h, most_w, borderwidth)
+            core.CopyArea(pixmap, border, gc, 0, borderwidth, most_w, 0, borderwidth, most_h)
+            core.ChangeWindowAttributes(self.wid, xcffib.xproto.CW.BorderPixmap, [border])
 
 
 class _Window:
@@ -448,6 +501,7 @@ class _Window:
             self._y = g.y
             self._width = g.width
             self._height = g.height
+            self._depth = g.depth
         except xcffib.xproto.DrawableError:
             # Whoops, we were too early, so let's ignore it for now and get the
             # values on demand.
@@ -455,6 +509,7 @@ class _Window:
             self._y = None
             self._width = None
             self._height = None
+            self._depth = None
 
         self.float_x: Optional[int] = None
         self.float_y: Optional[int] = None
@@ -493,6 +548,10 @@ class _Window:
         fset=_geometry_setter("height"),
         fget=_geometry_getter("height"),
     )
+    depth = property(
+        fset=_geometry_setter("depth"),
+        fget=_geometry_getter("depth"),
+    )
 
     @property
     def wid(self):
@@ -591,7 +650,6 @@ class _Window:
 
         state = self.window.get_net_wm_state()
 
-        logger.debug('_NET_WM_STATE: %s', state)
         for s in triggered:
             setattr(self, s, (s in state))
 
@@ -689,7 +747,8 @@ class _Window:
     def hide(self):
         # We don't want to get the UnmapNotify for this unmap
         with self.disable_mask(EventMask.StructureNotify):
-            self.window.unmap()
+            with self.qtile.core.disable_unmap_events():
+                self.window.unmap()
         self.hidden = True
 
     def unhide(self):
@@ -713,6 +772,29 @@ class _Window:
             eventmask=self._window_mask
         )
 
+    def _grab_click(self):
+        # Grab button 1 to focus upon click when unfocussed
+        for amask in self.qtile.core._auto_modmasks():
+            self.qtile.core.conn.conn.core.GrabButton(
+                True,
+                self.window.wid,
+                EventMask.ButtonPress,
+                xcffib.xproto.GrabMode.Sync,
+                xcffib.xproto.GrabMode.Async,
+                xcffib.xproto.Atom._None,
+                xcffib.xproto.Atom._None,
+                1,
+                amask,
+            )
+
+    def _ungrab_click(self):
+        # Ungrab button 1 when focussed
+        self.qtile.core.conn.conn.core.UngrabButton(
+            xcffib.xproto.Atom.Any,
+            self.window.wid,
+            xcffib.xproto.ModMask.Any,
+        )
+
     def get_pid(self):
         return self.window.get_net_wm_pid()
 
@@ -723,16 +805,16 @@ class _Window:
 
         Parameters
         ==========
-        x : int
-        y : int
-        width : int
-        height : int
-        borderwidth : int
-        bordercolor : string
-        above : bool, optional
-        margin : int or list, optional
+        x: int
+        y: int
+        width: int
+        height: int
+        borderwidth: int
+        bordercolor: string
+        above: bool, optional
+        margin: int or list, optional
             space around window as int or list of ints [N E S W]
-        above : bool, optional
+        above: bool, optional
             If True, the geometry will be adjusted to respect hints provided by the
             client.
         """
@@ -818,7 +900,7 @@ class _Window:
         self.borderwidth = width
         self.bordercolor = color
         self.window.configure(borderwidth=width)
-        self.window.paint_borders(color)
+        self.window.paint_borders(self.depth, color, width, self.width, self.height)
 
     def send_configure_notify(self, x, y, width, height):
         """Send a synthetic ConfigureNotify"""
@@ -916,17 +998,23 @@ class _Window:
                 state.remove(atom)
                 self.window.set_property('_NET_WM_STATE', state)
 
+        # re-grab button events on the previously focussed window
+        old = self.qtile.core._root.get_property("_NET_ACTIVE_WINDOW", 'WINDOW', unpack=int)
+        if old and old[0] in self.qtile.windows_map:
+            old_win = self.qtile.windows_map[old[0]]
+            if not isinstance(old_win, base.Internal):
+                old_win._grab_click()
         self.qtile.core._root.set_property("_NET_ACTIVE_WINDOW", self.window.wid)
+        self._ungrab_click()
+
+        if self.group:
+            self.group.current_window = self
         hook.fire("client_focus", self)
 
     def cmd_focus(self, warp: bool = True) -> None:
         """Focuses the window."""
         self.focus(warp)
 
-    def cmd_info(self):
-        """Returns a dictionary of info for this object"""
-        return self.info()
-
     def cmd_hints(self):
         """Returns the X11 hints (WM_HINTS and WM_SIZE_HINTS) for this window."""
         return self.hints
@@ -1054,29 +1142,15 @@ class Static(_Window, base.Static):
         self.conf_y = y
         self.conf_width = width
         self.conf_height = height
-        x = x or 0
-        y = y or 0
+        x = x or self.x
+        y = y or self.y
         self.x = x + screen.x
         self.y = y + screen.y
-        self.width = width or 0
-        self.height = height or 0
         self.screen = screen
-        self.place(self.x, self.y, self.width, self.height, 0, 0)
+        self.place(self.x, self.y, width or self.width, height or self.height, 0, 0)
+        self.unhide()
         self.update_strut()
-
-        # Grab button 1 to focus upon click
-        for amask in self.qtile.core._auto_modmasks():
-            self.qtile.core.conn.conn.core.GrabButton(
-                True,
-                self.window.wid,
-                EventMask.ButtonPress,
-                xcffib.xproto.GrabMode.Sync,
-                xcffib.xproto.GrabMode.Async,
-                xcffib.xproto.Atom._None,
-                xcffib.xproto.Atom._None,
-                1,
-                amask,
-            )
+        self._grab_click()
 
     def handle_ConfigureRequest(self, e):  # noqa: N802
         cw = xcffib.xproto.ConfigWindow
@@ -1153,8 +1227,6 @@ class Window(_Window, base.Window):
         EventMask.PropertyChange | \
         EventMask.EnterWindow | \
         EventMask.FocusChange
-    # Set when this object is being retired.
-    defunct = False
 
     def __init__(self, window, qtile):
         _Window.__init__(self, window, qtile)
@@ -1177,20 +1249,7 @@ class Window(_Window, base.Window):
         # add window to the save-set, so it gets mapped when qtile dies
         qtile.core.conn.conn.core.ChangeSaveSet(SetMode.Insert, self.window.wid)
         self.update_wm_net_icon()
-
-        # Grab button 1 to focus upon click
-        for amask in self.qtile.core._auto_modmasks():
-            self.qtile.core.conn.conn.core.GrabButton(
-                True,
-                self.window.wid,
-                EventMask.ButtonPress,
-                xcffib.xproto.GrabMode.Sync,
-                xcffib.xproto.GrabMode.Async,
-                xcffib.xproto.Atom._None,
-                xcffib.xproto.Atom._None,
-                1,
-                amask,
-            )
+        self._grab_click()
 
     @property
     def group(self):
@@ -1324,7 +1383,14 @@ class Window(_Window, base.Window):
     def toggle_minimize(self):
         self.minimized = not self.minimized
 
-    def cmd_static(self, screen=None, x=None, y=None, width=None, height=None):
+    def cmd_static(
+        self,
+        screen: Optional[int] = None,
+        x: Optional[int] = None,
+        y: Optional[int] = None,
+        width: Optional[int] = None,
+        height: Optional[int] = None,
+    ) -> None:
         """Makes this window a static window, attached to a Screen
 
         If any of the arguments are left unspecified, the values given by the
@@ -1341,6 +1407,7 @@ class Window(_Window, base.Window):
             self.group.remove(self)
         s = Static(self.window, self.qtile, screen, x, y, width, height)
         self.qtile.windows_map[self.window.wid] = s
+        self.qtile.core.update_client_list(self.qtile.windows_map)
         hook.fire("client_managed", s)
 
     def tweak_float(self, x=None, y=None, dx=0, dy=0,
@@ -1562,33 +1629,33 @@ class Window(_Window, base.Window):
         elif atoms["_NET_ACTIVE_WINDOW"] == opcode:
             source = data.data32[0]
             if source == 2:  # XCB_EWMH_CLIENT_SOURCE_TYPE_NORMAL
-                logger.info("Focusing window by pager")
+                logger.debug("Focusing window by pager")
                 self.qtile.current_screen.set_group(self.group)
                 self.group.focus(self)
             else:  # XCB_EWMH_CLIENT_SOURCE_TYPE_OTHER
                 focus_behavior = self.qtile.config.focus_on_window_activation
                 if focus_behavior == "focus":
-                    logger.info("Focusing window")
+                    logger.debug("Focusing window")
                     self.qtile.current_screen.set_group(self.group)
                     self.group.focus(self)
                 elif focus_behavior == "smart":
                     if not self.group.screen:
-                        logger.info("Ignoring focus request")
+                        logger.debug("Ignoring focus request")
                         return
                     if self.group.screen == self.qtile.current_screen:
-                        logger.info("Focusing window")
+                        logger.debug("Focusing window")
                         self.qtile.current_screen.set_group(self.group)
                         self.group.focus(self)
                     else:  # self.group.screen != self.qtile.current_screen:
-                        logger.info("Setting urgent flag for window")
+                        logger.debug("Setting urgent flag for window")
                         self.urgent = True
                 elif focus_behavior == "urgent":
-                    logger.info("Setting urgent flag for window")
+                    logger.debug("Setting urgent flag for window")
                     self.urgent = True
                 elif focus_behavior == "never":
-                    logger.info("Ignoring focus request")
+                    logger.debug("Ignoring focus request")
                 else:
-                    logger.warning("Invalid value for focus_on_window_activation: {}".format(focus_behavior))
+                    logger.debug("Invalid value for focus_on_window_activation: {}".format(focus_behavior))
         elif atoms["_NET_CLOSE_WINDOW"] == opcode:
             self.kill()
         elif atoms["WM_CHANGE_STATE"] == opcode:
@@ -1598,11 +1665,10 @@ class Window(_Window, base.Window):
             elif state == IconicState and self.qtile.config.auto_minimize:
                 self.minimized = True
         else:
-            logger.info("Unhandled client message: %s", atoms.get_name(opcode))
+            logger.debug("Unhandled client message: %s", atoms.get_name(opcode))
 
     def handle_PropertyNotify(self, e):  # noqa: N802
         name = self.qtile.core.conn.atoms.get_name(e.atom)
-        logger.debug("PropertyNotifyEvent: %s", name)
         if name == "WM_TRANSIENT_FOR":
             pass
         elif name == "WM_HINTS":
@@ -1638,7 +1704,7 @@ class Window(_Window, base.Window):
             # self.update_state()
             self.update_state()
         else:
-            logger.info("Unknown window property: %s", name)
+            logger.debug("Unknown window property: %s", name)
         return False
 
     def _items(self, name: str) -> ItemT:
